#!/usr/bin/python3

# Rev A.01.08 - May 9, 2023
#	* Forces "X" in upper right corner to call normal exit routine
#	* Enlarges button text
#	* Adds code for general window re-size (commented out for now)
# Rev A.01.09 - May 28, 2023
#	* Adds init code to force read of pots - requires A.01.01 of uP firmware but is backwards compatible with old rev
#   * Removed resizing code

# The DZ Company, LLC
# This software may be distributed freely as long as credit is given to DZKit

import os
import sw_rx_globals as g
from tkinter import *
from tkinter import messagebox
import tkinter.font
import queue
import subprocess
import time
import threading
import numpy as np
from array import array
from gpiozero import LED
import RPi.GPIO as GPIO
from stream_sdr_to_audio import demod, audio_callback
from stream_sdr_to_audio import	init_filts, init_audio, init_sdr
from stream_sdr_to_audio import	read_sdr_thread, shut_down_streams
from stream_sdr_to_audio import Filter_coeff
import SoapySDR
from SoapySDR import *

labels = []

# LDAV callback pseudocode

# if LDAV == 1
#	reset all data bits to input
#	LDAC = 1
# if LDAV == 0 and LWR == 0 (WRITE)
# 	if hs_mode=0 
#		cmd = data
#		hs_mode = 1
#		if cmd = 16: 
#			hs_mode = 2 (read)
# 	else if hs_mode=1
#		hs_mode = 0
#		read 10-bit data
#		if cmd == 0: AFG = data
#		if cmd == 1: RFG = data
#		if cmd == 2: RPG = data 01xxxxxxxx for clockwise, 11xxxxxxxx for ccw
# if hs_mode = 2 AND LWR = 1 (READ)
#	hs_mode = 0
#	if data = 16 
#		output keyer_mode
#	else (not used)
# LDAC = 0

def LDAV(self):
	LDAV_guts()
	
def LDAV_guts():
	
	if GPIO.input(26) == 1: # and GPIO.input(27) == 0: #LDAV = 1 and LWR = 0
		#print("LDAV = ",GPIO.input(26))
		#print("LWR = ",GPIO.input(27))
		#print("LDAC = ",GPIO.input(25))	
		#print("LDAV went high, config for input, setting LDAC high")
		GPIO.setup(15,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(16,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(17,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(18,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(19,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(20,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(21,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(22,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(23,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.setup(24,GPIO.IN, pull_up_down=GPIO.PUD_UP)
		GPIO.output(25,GPIO.HIGH) # drive LDAC high
		g.t1 = time.perf_counter()

	else: # falling edge
		g.t0 = time.perf_counter()	#reset time check
		#print ("LDAV went low")
		if g.hs_mode == 0: #and GPIO.input(27) == 0:
			#print ("  LWR was low")
			g.cmd = 0
			for i in range(24,14,-1): # pin 24 down to 15
				g.cmd = g.cmd*2 + GPIO.input(i)
			#print ("  mode = 0; cmd = ", g.cmd)
			# if g.cmd == 0, 1 or 2, next 10-bit data is afg, rfg or rpg, respectively
			# if g.cmd == 16 (read), expect 10-bit write of dummy data which we ignore
			g.hs_mode = 1		# so advance to hs_mode = 1
			if g.cmd > 16: 		# currently not allowed; probably out of sync
				#print ("Ignored command; setting mode and cmd back to 0")
				g.hs_mode = 0 	# no other cmds coded; reset to 0
				g.cmd = 0
		elif g.hs_mode == 1: #and GPIO.input(27) == 0:
			#print ("  LWR was low")
			data = 0
			for i in range(24,14,-1): # pin 24 down to 15
				data = data*2 + GPIO.input(i)
			#print ("  mode = 1; data = ", data)
			g.hs_mode = 0	# assume write op; changed if cmd = 16
			if g.cmd == 0:
				if muteButton["text"] != "Muted": 
					g.afg = data				# volume
				else:
					g.prev_afg = data			# store new value while muted, but don't change it live
			elif g.cmd == 1: 
				g.rfg = data
				gain = int (59.0 - (39.0*g.rfg/1024.0))	
				if g.agc == "Off": 
					#print ("Setting IFGR to: ", gain)
					sdr.setGain(SOAPY_SDR_RX, 0, "IFGR", gain)
				else:
					sp = -60 + (60 * g.rfg / 1024.0)
					#print ("AGC setpoint = ", sp)
					sdr.writeSetting('agc_setpoint', sp)
			elif g.cmd == 2: 
				g.rpg = data
				canvas.delete(g.line)
				if g.rpg == 768:
					g.center_freq += g.freq_inc
				else:
					g.center_freq -= g.freq_inc
				if g.center_freq < g.currband_start:
					g.center_freq = g.currband_start
				if g.center_freq > g.currband_start + g.currband_span:
					g.center_freq = g.currband_start + g.currband_span
				rclFreq()
				freqBox()
			else:	# read
				dummy = data	# ignore dummy data
				g.hs_mode = 2	# expect read op
				#print ("  Ignoring dummy data. Setting mode to read")
		else: # hs_mode = 2 (read)
			g.hs_mode = 0
			if GPIO.input(27) == 1: # only if LWR is high
				#print ("  mode = 2; LWR high; setting up for output")
				GPIO.setup(15,GPIO.OUT)
				GPIO.setup(16,GPIO.OUT)
				GPIO.setup(17,GPIO.OUT)
				GPIO.setup(18,GPIO.OUT)
				GPIO.setup(19,GPIO.OUT)
				GPIO.setup(20,GPIO.OUT)
				GPIO.setup(21,GPIO.OUT)
				GPIO.setup(22,GPIO.OUT)
				GPIO.setup(23,GPIO.OUT)
				GPIO.setup(24,GPIO.OUT)
			
				if g.cmd == 16:	# only supported read cmd now
					data = g.keyer_mode # 0=IambicA,1=IambicB,2=Ultimatic,3 =Manual
					if g.nrun == 1:
						data += 128	# set sign bit to force read of initial pot settings
						nrun = 0
				else:  # add more cases here (17,18...)
					data = 0
				s = format (data,'b').zfill(10) # convert int data to binary string
				#print ("Outputting data: Data = ", s)
				for i in range(15,25):
					if s[24-i] == '0':
						ch = 0
					else:
						ch = 1
					GPIO.output(i,ch) # output one bit at a time	
		#print ("Setting LDAC low")	
		GPIO.output(25,GPIO.LOW)	# drive LDAC low to continue data handshake

def muteToggle():
	if muteButton["text"]=="Muted":
		muteButton["text"]="Receiving"
		g.afg = g.prev_afg
	else:
		muteButton["text"]="Muted"
		g.prev_afg = g.afg
		g.afg = 0				

def agcToggle():
	if g.agc == "On":
		g.agc = "Off"
		sdr.setGainMode(SOAPY_SDR_RX, 0, False)		# turn off agc
		gr_check()
		gain = int (59.0 - (39.0*g.rfg/1024.0))		# 20->59
		sdr.setGain(SOAPY_SDR_RX, 0, "IFGR", gain)	# set ifgr
	else:
		g.agc = "On"
		sdr.setGain(SOAPY_SDR_RX, 0, "IFGR", 20)	# set gain to 20
		sdr.setGainMode(SOAPY_SDR_RX, 0, True)
		sp = -60 + (60 * g.rfg / 1024.0)
		#print ("AGC setpoint = ", sp)
		sdr.writeSetting('agc_setpoint', sp)
	agcButton["text"] = "AGC "+ str(g.agc)

def gr_check():
	g.gr = g.lna*6	# ok for g.lna <= 3
	if g.center_freq < 60000000.0:
		if g.lna == 4: 
			g.gr = 37
		elif g.lna == 5: 
			g.gr = 42
		elif g.lna > 5:	# gr was set higher while on a freq > 60MHz - set to 12dB
			g.gr = 12
			g.lna = 2
	else:
		if g.lna == 4: g.gr = 20
		if g.lna == 5: g.gr = 26
		if g.lna == 6: g.gr = 32
		if g.lna == 7: g.gr = 38
		if g.lna == 8: g.gr = 57
		if g.lna == 9: g.gr = 62
	grButton["text"] = "Att " + str(g.gr) + "dB"
	sdr.setGain(SOAPY_SDR_RX, 0, "RFGR", g.lna)
	i = 0
	t0 = time.perf_counter()
	while time.perf_counter() - t0 < .1:			# wait 100 ms for gain to "take"
		i += 1

def gr_right_click(event):
	# 0-5 (up to 60MHz), else 0-9 (LNAState)
	g.lna -= 1
	if g.center_freq < 60000000.0:
		if g.lna < 0: g.lna = 5
	else:
		if g.lna < 0: g.lna = 9
	gr_check()


def grToggle():
	# 0-5 (up to 60MHz), else 0-9 (LNAState)
	g.lna += 1
	if g.center_freq < 60000000.0:
		if g.lna > 5: g.lna = 0
	else:
		if g.lna > 9: g.lna = 0
	gr_check()

def prevPage():
	# Save current settings
	if g.page == 1:
		g.selected_band_p1 = g.selected_band
	elif g.page == 2:
		g.selected_band_p2 = g.selected_band
	else:
		g.selected_band_p3 = g.selected_band	
	
	# Decrement page
	g.page -=1
	if g.page == 0:
		g.page = 3
	
	# Set new parameters
	g.currband_span = 3e6
	if g.page == 1:
		cb, cf, mode = g.band_info_p1[g.selected_band_p1]
		g.selected_band = g.selected_band_p1
		if g.selected_band == 0: 
			g.currband_span = 1.8e6
	elif g.page == 2:
		cb, cf, mode = g.band_info_p2[g.selected_band_p2]
		g.selected_band = g.selected_band_p2
	else:
		cb, cf, mode = g.band_info_p3[g.selected_band_p3]
		g.selected_band = g.selected_band_p3

	# when changing freq's where lna's are incompatible, pick 12dB which works for both	
	if (cf >= 60.0e6 and g.center_freq < 60.0e6) or (cf < 60.0e6 and g.center_freq >= 60.0e6):
		g.lna = 2
		sdr.setGain(SOAPY_SDR_RX, 0, "RFGR", g.lna)

	g.currband_start = cb
	g.center_freq = cf
	g.mode = mode
		
	# Draw the new bands
	band_setup()
	
	# init dial pointer memory
	set_dps()
	
	# Set new freq, new mode and draw dial pointer
	modeButton["text"] = g.mode
	
	rclFreq()
	updMode()
	freqBox()
	gr_check()
	
	return
	
def nextPage():
	# Save current settings
	if g.page == 1:
		g.selected_band_p1 = g.selected_band
	elif g.page == 2:
		g.selected_band_p2 = g.selected_band		
	else:
		g.selected_band_p3 = g.selected_band
			
	# Increment page
	g.page +=1
	if g.page == 4:
		g.page = 1
	
	# Set new parameters
	g.currband_span = 3e6
	if g.page == 1:
		cb, cf, mode = g.band_info_p1[g.selected_band_p1]
		g.selected_band = g.selected_band_p1
		if g.selected_band == 0: 
			g.currband_span = 1.8e6
	elif g.page == 2:
		cb, cf, mode = g.band_info_p2[g.selected_band_p2]
		g.selected_band = g.selected_band_p2
	else:
		cb, cf, mode = g.band_info_p3[g.selected_band_p3]
		g.selected_band = g.selected_band_p3

	# when changing freq's where lna's are incompatible, pick 12dB which works for both
	if (cf >= 60.0e6 and g.center_freq < 60.0e6) or (cf < 60.0e6 and g.center_freq >= 60.0e6):
		sdr.setGain(SOAPY_SDR_RX, 0, "RFGR", 2)	

	g.currband_start = cb
	g.center_freq = cf
	g.mode = mode
		
	# Draw the new bands
	band_setup()
	
	# init dial pointer memory
	set_dps()
	
	# Set new freq, show new mode and draw dial pointer
	modeButton["text"] = g.mode

	rclFreq()
	updMode()
	freqBox()
	gr_check()
	
	return
	
def updMode():
	if g.mode == "FM":
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 200e3)
	elif g.mode == "NBFM":
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 12.5e3)
	elif g.mode == "USB" or g.mode == "LSB":
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 4e3)
	elif g.mode == "CW(USB)" or g.mode == "CW(LSB}":
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 1e3)
	else:	# AM
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 8e3)

	
def Mode():
	if modeButton["text"] == "AM":
		modeButton["text"] = "FM"
		g.mode = "FM"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 200e3)
	elif modeButton["text"] == "FM":
		modeButton["text"] = "NBFM"
		g.mode = "NBFM"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 12.5e3)
	elif modeButton["text"] == "NBFM":
		modeButton["text"] = "USB"
		g.mode = "USB"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 4e3)
	elif modeButton["text"] == "USB":
		modeButton["text"] = "LSB"
		g.mode = "LSB"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 4e3)
	elif modeButton["text"] == "LSB":
		modeButton["text"] = "CW(USB)"
		g.mode = "CW(USB)"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 1e3)
	elif modeButton["text"] == "CW(USB)":
		modeButton["text"] = "CW(LSB)"
		g.mode = "CW(LSB)"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 1e3)
	else:
		modeButton["text"] = "AM"
		g.mode = "AM"
		sdr.setBandwidth(SOAPY_SDR_RX, 0, 8e3)
		
# get current settings, change only mode on band-by-band basis
	if g.page == 1:
		a, b, c = g.band_info_p1[g.selected_band]
		g.band_info_p1[g.selected_band] = a,b,g.mode
	elif g.page == 2:
		a, b, c = g.band_info_p2[g.selected_band]
		g.band_info_p2[g.selected_band] = a,b,g.mode
	else:
		a, b, c = g.band_info_p3[g.selected_band]
		g.band_info_p3[g.selected_band] = a,b,g.mode
	return
	
def Keyer():
	if keyerButton["text"] == "Iambic A":
		keyerButton["text"] = "Iambic B"
		g.keyer_mode = 1
	elif keyerButton["text"] == "Iambic B":
		keyerButton["text"] = "Ultimatic"
		g.keyer_mode = 2
	elif keyerButton["text"] == "Ultimatic":
		keyerButton["text"] = "Manual"
		g.keyer_mode = 3
	elif keyerButton["text"] == "Manual":
		keyerButton["text"] = "Iambic A"
		g.keyer_mode = 0
	return
	
def saveFreq():
	# Save current settings
	if g.page == 1:
		g.band_info_p1[g.selected_band] = g.currband_start,g.center_freq,g.mode
		g.selected_band_p1 = g.selected_band
	elif g.page == 2:
		g.band_info_p2[g.selected_band] = g.currband_start,g.center_freq,g.mode
		g.selected_band_p2 = g.selected_band		
	else:
		g.band_info_p3[g.selected_band] = g.currband_start,g.center_freq,g.mode
		g.selected_band_p3 = g.selected_band
	canvas.delete(g.orangeline[g.selected_band])
	set_dps()
	rclFreq()
	return

def fi_right_click(event):
	# set frequency increment in reverse mode
	if g.freq_inc == 1:
		g.freq_inc = 10000
	elif g.freq_inc == 10:
		g.freq_inc = 1			
	elif g.freq_inc == 100:
		g.freq_inc = 10
	elif g.freq_inc == 1000:
		g.freq_inc = 100
	elif g.freq_inc == 5000:
		g.freq_inc = 1000
	else:
		g.freq_inc = 5000
	set_fi()

def freqInc():
	# set frequency increment
	if g.freq_inc == 1:
		g.freq_inc = 10
	elif g.freq_inc == 10:
		g.freq_inc = 100			
	elif g.freq_inc == 100:
		g.freq_inc = 1000
	elif g.freq_inc == 1000:
		g.freq_inc = 5000
	elif g.freq_inc == 5000:
		g.freq_inc = 10000
	else:
		g.freq_inc = 1
	set_fi()
	
def set_fi():
	freqincButton["text"] = "Inc " + str(g.freq_inc)
	g.center_freq = float (int(g.center_freq/g.freq_inc) * g.freq_inc)
	rclFreq()
	freqBox()
	
def shut_down_linux():
	os.system("sudo shutdown now -h")
	return
	
def leave():
	res=messagebox.askyesnocancel('Exit','Yes: shut down receiver\nNo: exit to OS')
	if res == True:
		# Clear screen
		canvas.delete('all')
		clear_labels()
		if g.freq_box_exists:
			g.lF1.place_forget()
		lF1 = Label(canvas, text='Shutting down receiver. Wait 15 seconds before removing power', bg="black",  fg="yellow", font=('Arial',18))
		lF1.place(x=180, y=250)
		win.after(5000, shut_down_linux)
	elif res == False:
		g.stopped = True		# stop sdr callback
		read_signals.join()		# wait for it to finish
		#print ("sdr callback exited")
		GPIO.remove_event_detect(26)	# stop handshake callback
		#print ("handshake callback ended")
		shut_down_streams (sdr)		# stop streaming from sdr and audio
		#print ("Done shutting down callbacks and streams")
		# Clear screen
		canvas.delete('all')
		#print ("Done deleting all canvas items")
		clear_labels()
		#print ("Done clearing labels")
		win.destroy()
		#print ("Done destroying window")
		os._exit(0)

			
def rclFreq():
	prev_afg = g.afg
	g.afg *= 0.5		# lower audio gain while changing frequency to eliminate clicks
	t0 = time.perf_counter()
	sdr.setFrequency(SOAPY_SDR_RX, 0, int(g.center_freq))
	i = 0
	while time.perf_counter()-t0 < 0.001:	# wait 1ms for frequency to be set
		i += 1
	g.afg = prev_afg
	dp1 = g.xband_offset + int (922 * (g.center_freq - g.currband_start)/g.currband_span)
	ytop = g.ytop[g.selected_band]
	ybot = ytop + 25
	x = g.line
	g.line = canvas.create_line(dp1, ytop, dp1, ybot, fill="white",width=2)	# create new line before deleting old one
	canvas.delete(x)
	return
	
def set_dps():
	for i in range(8):
		span = 3.0e6
		if g.page == 1:
			if i == 0:
				span = 1.8e6
			cb, cf, m = g.band_info_p1[i]
		elif g.page == 2:
			cb, cf, m = g.band_info_p2[i]
		else:
			cb, cf, m = g.band_info_p3[i]
		dp = g.xband_offset + int (922.0 * (cf - cb)/span)
		ytop = g.ytop[i]
		ybot = ytop + 25
		g.orangeline[i] = canvas.create_line(dp, ytop, dp, ybot, fill="orange",width=2)
	return
	
def canvas_click (event):
	if str(event.widget) == '.!canvas' and event.y < 500:
		new_sel_band = int(event.y / 65)	# 0 to 7
		span = 3e6	# assume 3MHz
		if g.page == 1:
			if new_sel_band == 0:
				span = 1.8e6	# only band with less than 3MHz span
			cb, cf, m = g.band_info_p1[new_sel_band]
		elif g.page == 2:
			cb, cf, m = g.band_info_p2[new_sel_band]
		elif g.page == 3:
			cb, cf, m = g.band_info_p3[new_sel_band]
		# if "clicked" freq is on a different band, then go to the saved freq
		# else go to the nearest 10kHz point
		if new_sel_band != g.selected_band:
			# when changing freq's where lna's are incompatible, pick 12dB which works for both	
			if (cf >= 60.0e6 and g.center_freq < 60.0e6) or (cf < 60.0e6 and g.center_freq >= 60.0e6):
				g.lna = 2
				sdr.setGain(SOAPY_SDR_RX, 0, "RFGR", g.lna)	
			g.currband_start = cb
			g.center_freq = cf
			g.mode = m
			g.selected_band = new_sel_band
			g.currband_span = span
			modeButton["text"] = g.mode
			rclFreq()
			updMode()
		else:
			curr_dp = g.xband_offset + int (922.0 * (g.center_freq - cb)/span)
			cf = cb + span*(event.x - g.xband_offset)/922.0
			if cf < cb:
				cf = cb
			if cf > cb + g.currband_span:
				cf = cb + g.currband_span
			cf = 10000.0 * int(cf/10000.0)	# nearest 10kHz
			g.center_freq = cf
			rclFreq()
		freqBox()	# show frequency
		gr_check()
	return

def close():
	#GPIO.setmode()
	shut_down_streams (sdr)
	win.destroy()

def xy(el,xel):
	if xel: 
		r=el*g.xscale
	else: 
		r=el*g.yscale
	return r

def clear_labels():
	for item in labels:
		item.place_forget()
	labels.clear()
	
def band(x0,x1,c1,x2,c2,x3,c3,x4,c4,x5,c5,x6,c6,y1):
	lbl = Label(canvas, text=x0, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=5, y=y1-3)
	labels.append(lbl)
	
	a1 = canvas.create_rectangle(0, 0, x1, 10, fill=c1, outline='white')
	a2 = canvas.create_rectangle(0, 0, x2, 10, fill=c2, outline='white')
	a3 = canvas.create_rectangle(0, 0, x3, 10, fill=c3, outline='white')
	a4 = canvas.create_rectangle(0, 0, x4, 10, fill=c4, outline='white')
	a5 = canvas.create_rectangle(0, 0, x5, 10, fill=c5, outline='white')
	a6 = canvas.create_rectangle(0, 0, x6, 10, fill=c6, outline='white')

	xx = 45
	canvas.move(a1, xx, y1)  #45
	xx = xx + x1
	canvas.move(a2, xx, y1)  #160 = 45 + x1
	xx = xx + x2
	canvas.move(a3, xx, y1)  #280 = 45 + x1 + x2
	xx = xx + x3
	canvas.move(a4, xx, y1)  #400 = 45 + x1 + x2 + x3
	xx = xx + x4
	canvas.move(a5, xx, y1)  #460 = 45 + x1 + x2 + x3 + x4
	xx = xx + x5
	canvas.move(a6, xx, y1)  #640 = 45 + x1 + x2 + x3 + x4 + x5
	
	lbl = Label(canvas, text=x0, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=int(768*g.xscale), y=y1-int(3*g.yscale))
	labels.append(lbl)
	return

def greenband(fput,fwid,y,start,span):
	# Create a green box fwid (Hz) long and 9 pixels high, no border
	x = int(922.0 * fwid / span)
	x1 = int(922.0 * (fput-start) / span)
	xadj = 963 - (g.xband_offset + x + x1 + 1)
	if xadj < 0: x += xadj
	a = canvas.create_rectangle(0, 0, x, 9, fill="green", outline='white', width=0)
	# Move it to its destination, fput,y (upper left corner) (+1 to account for border)
	canvas.move(a, g.xband_offset + x1 + 1, y+1)
	return
	
def band_freq(x1,f1,x2,f2,x3,f3,x4,f4,x5,f5,x6,f6,x7,f7,row_top):
	lbl = Label(canvas, text=f1, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x1, y=row_top)
	labels.append(lbl)
	
	lbl = Label(canvas, text=f2, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x2, y=row_top)
	labels.append(lbl)
	
	lbl = Label(canvas, text=f3, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x3, y=row_top)
	labels.append(lbl)
	
	lbl = Label(canvas, text=f4, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x4, y=row_top)
	labels.append(lbl)
	
	lbl = Label(canvas, text=f5, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x5, y=row_top)
	labels.append(lbl)
		
	lbl = Label(canvas, text=f6, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x6, y=row_top)
	labels.append(lbl)
	
	lbl = Label(canvas, text=f7, bg="black",  fg="white", font=('Arial',9,'bold'))
	lbl.place(x=x7, y=row_top)
	labels.append(lbl)
	return

def freqBox():
	if g.freq_box_exists:
		g.lF1.place_forget()
	# test case f = 101145123
	s0 = int((g.center_freq + g.frq_ofs)/ 1e6)	# s0 = 101
	s1 = (g.center_freq + g.frq_ofs) - (s0 * 1e6)	# s1 = 101145123 - 101000000 = 145123
	s2 = int(s1/1000)				# s2 = 145
	s3 = int(s1 - (s2 * 1e3))		# s3 = 145123 - 145000 = 123
	if s0 == 0: 
		supr = "   "
	elif s0 > 1 and s0 < 10:
		supr = "  " + "{0:>1}".format(str(s0))
	elif s0 > 10 and s0 < 100:
		supr = " " + "{0:>2}".format(str(s0))
	else:
		supr = "{0:>3}".format(str(s0))
	s = (supr + " " + 
		"{0:0>3}".format(str(s2)) + " " +
		"{0:0>3}".format(str(s3)))
	
	name = g.get_freq_name()
	if name != -1:
		s = s + " " + name
	g.lF1 = Label(canvas, text=s, bg="black",  fg="yellow", font=('Arial',22,'bold'))

	if g.center_freq + g.frq_ofs < 1e6:
		xpos = 420
	elif g.center_freq + g.frq_ofs >= 1e6 and g.center_freq + g.frq_ofs < 10e6:
		xpos = 430
	else:
		xpos = 431
	g.lF1.place(x=xpos, y=298)
	g.freq_box_exists = True
	return 
	
# created for 800x480 display, now 1024 x 600, so mpy x by g.xscale, y by g.yscale and truncate
def band_setup():
	# Clear screen
	canvas.delete('all')
	clear_labels()
	
	x = 120*g.xscale
	x23 = 23 * g.xscale
	x143 = 143 * g.xscale
	x263 = 263 * g.xscale
	x383 = 383 * g.xscale
	x503 = 503 * g.xscale
	x623 = 623 * g.xscale
	x740 = 740 * g.xscale
	x743 = 743 * g.xscale
	x749 = 749 * g.xscale

	if g.page==1:
		band("kHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(25, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(75, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(125, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(175, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(225, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(275, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(325, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(375, False))

		band_freq(xy(26, True),'200',xy(146, True),'500',xy(266, True),'800',xy(383, True),'1100',xy(503, True),'1400',xy(623, True),'1700',xy(743, True),'2000',xy(5, False))
		band_freq(xy(29, True),'2.0',xy(149, True),'2.5',xy(269, True),'3.0',xy(389, True),'3.5',xy(509, True),'4.0',xy(629, True),'4.5',xy(746, True),'5.0',xy(55, False))
		band_freq(xy(29, True),'5.0',xy(149, True),'5.5',xy(269, True),'6.0',xy(389, True),'6.5',xy(509, True),'7.0',xy(629, True),'7.5',xy(746, True),'8.0',xy(105, False))
		band_freq(xy(29, True),'8.0',xy(149, True),'8.5',xy(269, True),'9.0',xy(389, True),'9.5',xy(503, True),'10.0',xy(623, True),'10.5',xy(743, True),'11.0',xy(155, False))
		band_freq(xy(23, True),'11.0',xy(143, True),'11.5',xy(263, True),'12.0',xy(383, True),'12.5',xy(503, True),'13.0',xy(623, True),'13.5',xy(743, True),'14.0',xy(205, False))
		band_freq(xy(23, True),'14.0',xy(143, True),'14.5',xy(263, True),'15.0',xy(383, True),'15.5',xy(503, True),'16.0',xy(623, True),'16.5',xy(743, True),'17.0',xy(255, False))
		band_freq(xy(23, True),'17.0',xy(143, True),'17.5',xy(263, True),'18.0',xy(383, True),'18.5',xy(503, True),'19.0',xy(623, True),'19.5',xy(743, True),'20.0',xy(305, False))
		band_freq(xy(23, True),'20.0',xy(143, True),'20.5',xy(263, True),'21.0',xy(383, True),'21.5',xy(503, True),'22.0',xy(623, True),'22.5',xy(743, True),'23.0',xy(355, False))

		row_bottom = xy(38, False)
		lbl = Label(canvas, text="AERO, MARINE, NAV, EXP", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(35, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (200e3,300e3,xy(25, False),200e3,1.8e6)
		
		lbl = Label(canvas, text="STANDARD BROADCAST", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(344, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="AMATEUR (160M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(674, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (1.8e6,200e3,xy(25, False),200e3,1.8e6)
		
		row_bottom = int(88 * g.yscale)
		lbl = Label(canvas, text="120M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(115, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (2.3e6,200e3,xy(75, False),2e6,3e6)
		
		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(145, True), y=row_bottom)
		labels.append(lbl)

		lbl = Label(canvas, text="CHU", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(342, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="90M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(318, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (3.2e6,200e3,xy(75, False),2e6,3e6)
		
		lbl = Label(canvas, text="AMATEUR (80M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(417, True), y=row_bottom)
		labels.append(lbl)
				
		greenband (3.5e6,500e3,xy(75, False),2e6,3e6)

		lbl= Label(canvas, text="60M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(719, True), y=row_bottom)
		labels.append(lbl)
				
		greenband (4.75e6,250e3,xy(75, False),2e6,3e6)

		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(743, True), y=row_bottom)
		labels.append(lbl)
		
		row_bottom = int(138 * g.yscale)
		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(22, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="AMATEUR (60M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(83, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (5.33e6,70e3,xy(125, False),5e6,3e6)
		
		lbl = Label(canvas, text="49M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(283, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (5.95e6,250e3,xy(125, False),5e6,3e6)

		lbl = Label(canvas, text="AMATEUR (40M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(513, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (7e6,300e3,xy(125, False),5e6,3e6)
	
		lbl = Label(canvas, text="41M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(615, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="CHU", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(707, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (7.31e6,290e3,xy(125, False),5e6,3e6)

		row_bottom = int(188 * g.yscale)
		lbl = Label(canvas, text="MARINE & WEATHER", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(108, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (8e6,1e6,xy(175, False),8e6,3e6)

		lbl = Label(canvas, text="31M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(400, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (9.2e6,700e3,xy(175, False),8e6,3e6)

		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(502, True), y=row_bottom)
		labels.append(lbl)

		lbl = Label(canvas, text="AMATEUR (30M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(536, True), y=row_bottom)
		labels.append(lbl)		
		
		greenband (10.1e6,50e3,xy(175, False),8e6,3e6)

		row_bottom = int(238 * g.yscale)
		lbl = Label(canvas, text="25M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(240, True), y=row_bottom)
		labels.append(lbl)		
		
		greenband (11.6e6,600e3,xy(225, False),11e6,3e6)
		
		lbl = Label(canvas, text="MARINE", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(333, True), y=row_bottom-39)
		labels.append(lbl)
		
		greenband (12.29e6,75e3,xy(225, False),11e6,3e6)		
		
		lbl = Label(canvas, text="22M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(680, True), y=row_bottom)
		labels.append(lbl)	
		
		greenband (13.57e6,300e3,xy(225, False),11e6,3e6)
		
		row_bottom = int(288 * g.yscale)
		lbl = Label(canvas, text="AMATEUR (20M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(39, True), y=row_bottom)
		labels.append(lbl)		
		
		greenband (14e6,350e3,xy(275, False),14e6,3e6)
		
		lbl = Label(canvas, text="19M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(370, True), y=row_bottom)
		labels.append(lbl)	
		
		greenband (15.1e6,700e3,xy(275, False),14e6,3e6)
		
		lbl = Label(canvas, text="MARINE", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(610, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (16.42e6,120e3,xy(275, False),14e6,3e6)	
		
		lbl = Label(canvas, text="CHU", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(183, True), y=row_bottom)
		labels.append(lbl)

		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(262, True), y=row_bottom)
		labels.append(lbl)

		row_bottom = int(338 * g.yscale)
		lbl = Label(canvas, text="16M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(192, True), y=row_bottom)
		labels.append(lbl)	
		
		greenband (17.48e6,420e3,xy(325, False),17e6,3e6)
		
		lbl = Label(canvas, text="AMATEUR (17M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(264, True), y=row_bottom)
		labels.append(lbl)		
	
		greenband (18.068e6,100e3,xy(325, False),17e6,3e6)

		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(743, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="MARINE", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(455, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (18.825e6,18e3,xy(325, False),17e6,3e6)	
		
		lbl = Label(canvas, text="15M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(497, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (18.9e6,120e3,xy(325, False),17e6,3e6)

		row_bottom = int(388 * g.yscale)
		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(22, True), y=row_bottom)
		labels.append(lbl)
		
		lbl = Label(canvas, text="AMATEUR (15M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(290, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (21e6,450e3,xy(375, False),20e6,3e6)
		
		lbl = Label(canvas, text="13M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(423, True), y=row_bottom)
		labels.append(lbl)		
				
		greenband (21.455e6,395e3,xy(375, False),20e6,3e6)
		
		lbl = Label(canvas, text="MARINE", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(538, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (22.159e6,18e3,xy(375, False),20e6,3e6)	

	if g.page == 2:
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(25, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(75, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(125, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(175, False))
		band("MHz",x,'black',x,'black',x,'black',x,'black',x,'black',x,'black',xy(225, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(275, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(325, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(375, False))

		band_freq(x23,'23.0',x143,'23.5',x263,'24.0',x383,'24.5',x503,'25.0',x623,'25.5',x743,'26.0',xy(5, False))
		band_freq(x23,'26.0',x143,'26.5',x263,'27.0',x383,'27.5',x503,'28.0',x623,'28.5',x743,'29.0',xy(55, False))
		band_freq(x23,'29.0',x143,'29.5',x263,'30.0',x383,'30.5',x503,'31.0',x623,'31.5',x743,'32.0',xy(105, False))
		band_freq(x23,'50.0',x143,'50.5',x263,'51.0',x383,'51.5',x503,'52.0',x623,'52.5',x743,'53.0',xy(155, False))
		band_freq(x23,'53.0',x143,'53.5',x263,'54.0',x383,'54.5',x503,'55.0',x623,'55.5',x743,'56.0',xy(205, False))
		band_freq(x23,'75.0',x143,'75.5',x263,'76.0',x383,'76.5',x503,'77.0',x623,'77.5',x743,'78.0',xy(255, False))
		band_freq(x23,'78.0',x143,'78.5',x263,'79.0',x383,'79.5',x503,'80.0',x623,'80.5',x743,'81.0',xy(305, False))
		band_freq(x23,'81.0',x143,'81.5',x263,'82.0',x383,'82.5',x503,'83.0',x623,'83.5',x743,'84.0',xy(355, False))
		
		row_bottom = xy(38, False)
		lbl = Label(canvas, text="AMATEUR (12M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(424, True), y=row_bottom)
		labels.append(lbl)
	
		lbl = Label(canvas, text="WWV", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(503, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (24.89e6,100e3,xy(25, False),23e6,3e6)
		
		lbl = Label(canvas, text="MARINE", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(532, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (25.1e6,18e3,xy(25, False),23e6,3e6)	
		
		lbl = Label(canvas, text="11M", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(695, True), y=row_bottom)
		labels.append(lbl)

		greenband (25.6e6,400e3,xy(25, False),23e6,3e6)

		row_bottom = xy(88, False)
		lbl = Label(canvas, text="CB", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(313, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (26.965e6,440e3,xy(75, False),26e6,3e6)
		
		row_bottom = xy(88, False)
		lbl = Label(canvas, text="AMATEUR (10M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(600, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (28e6,1e6,xy(75, False),26e6,3e6)

		row_bottom = xy(138, False)
		lbl = Label(canvas, text="AMATEUR (10M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(85, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (29e6,700e3,xy(125, False),29e6,3e6)
		
		row_bottom = int(188 * g.yscale)
		lbl = Label(canvas, text="AMATEUR (6M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(358, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (50e6,3e6,xy(175, False),50e6,3e6)
		
		row_bottom = int(238 * g.yscale)
		lbl = Label(canvas, text="AMATEUR (6M)", bg="black",  fg="green", font=('Arial',9,'bold'))
		lbl.place(x=xy(122, True), y=row_bottom)
		labels.append(lbl)
		
		greenband (53e6,1e6,xy(225, False),53e6,3e6)

	elif g.page == 3:
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(25, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(75, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(125, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(175, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(225, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(275, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(325, False))
		band("MHz",x,'green',x,'black',x,'green',x,'black',x,'green',x,'black',xy(375, False))

		band_freq(x23,'84.0',x143,'84.5',x263,'85.0',x383,'85.5',x503,'86.0',x623,'86.5',x743,'87.0',xy(5, False))
		band_freq(x23,'87.0',x143,'87.5',x263,'88.0',x383,'88.5',x503,'89.0',x623,'89.5',x743,'90.0',xy(55, False))
		band_freq(x23,'90.0',x143,'90.5',x263,'91.0',x383,'91.5',x503,'92.0',x623,'92.5',x743,'93.0',xy(105, False))
		band_freq(x23,'93.0',x143,'93.5',x263,'94.0',x383,'94.5',x503,'95.0',x623,'95.5',x743,'96.0',xy(155, False))
		band_freq(x23,'96.0',x143,'96.5',x263,'97.0',x383,'97.5',x503,'98.0',x623,'98.5',x743,'99.0',xy(205, False))
		band_freq(x23,'99.0',x143,'99.5',x263,'100.0',x383,'100.5',x503,'101.0',x623,'101.5',x740,'102.0',xy(255, False))
		band_freq(x23,'102.0',x143,'102.5',x263,'103.0',x383,'103.5',x503,'104.0',x623,'104.5',x740,'105.0',xy(305, False))
		band_freq(x23,'105.0',x143,'105.5',x263,'106.0',x383,'106.5',x503,'107.0',x623,'107.5',x740,'108.0',xy(355, False))
	return


#
# Main program starts here
#
sdr = SoapySDR.Device(dict(driver="sdrplay"))

init_sdr(sdr)

init_filts(False)
	
g.num_rx_calls = int(np.floor(g.num_samps/g.max_buffer_size))
g.sample_buffer = np.array([0]*g.max_buffer_size, np.complex64)
g.x = np.array([0]*g.num_samps, np.complex64)   # new data to be read from SDR
g.ab_frame = np.array([0]*g.num_samps, np.int16) 

# set volume control to max - we control volume through array magnitude sent to ALSA
command = ["amixer", "sset", "Master", "{}%".format(g.volume)]
subprocess.Popen(command)

# set touchscreen to use only internal monitor
command = ["xinput", "list", "--id-only", """Matrix Orbital Multi-Touch Device"""]
process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
out, err = process.communicate()
if err.decode('ascii') == "":
	out = out.strip()
	out = out.decode('ascii')
	command = ["xinput", "map-to-output", out, "HDMI-1"]
	print (command)
	subprocess.run(command)

# Set up GPIO port for comm to Saguaro control board
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
# GPIO 14 = Fan
# GPIO 15-24 = D0-D9 10 bit data read/write - initialize to inputs
# GPIO 25 = LDAC output to uP (Data Accepted)
# GPIO 26 = LDAV input from uP (Data Valid)
# GPIO 27 = LWR input from uP (Read/Write)
#GPIO.setup(14,GPIO.OUT, initial=GPIO.LOW)
GPIO.setup(15,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(16,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(17,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(18,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(19,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(20,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(21,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(22,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(23,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(24,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(25,GPIO.OUT, initial=GPIO.HIGH)
GPIO.setup(26,GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(27,GPIO.IN, pull_up_down=GPIO.PUD_UP)

# GUI definition
win = Tk()  
win.geometry("1024x565")  # Allow room for title bar (35 pixels)    
win.title("DZKit Saguaro \u25C6 Shortwave Receiver")
myFont = tkinter.font.Font(family='Arial',size=9)
win.protocol('WM_DELETE_WINDOW', leave)

canvas = Canvas(win, width=1024, height=565, bg='black')
canvas.pack(anchor=CENTER, expand=True)

g.page = 1
g.selected_band = 0
g.currband_span = 1.8e6	# only band with less than 3MHz span
g.currband_start, g.center_freq, g.mode = g.band_info_p1[g.selected_band]

band_setup()

# init dial pointer memory
set_dps()

# init freq, active dial pointer
rclFreq()

# Display current freq above "inc" button
freqBox()

# Widgets			
muteButton = Button(canvas, text="Receiving", font=('Arial',12), command = muteToggle, bg = "bisque2", height = 1, width = 6)
muteButton.place(x=int(20*g.xscale), y=int(410*g.yscale))

agcButton = Button(canvas, text="AGC On", font=('Arial',12), command = agcToggle, bg = "bisque2", height = 1, width = 6)
agcButton.place(x=int(98*g.xscale), y=int(410*g.yscale))

grButton = Button(canvas, text="Att 12dB", font=('Arial',12), command = grToggle, bg = "bisque2", height = 1, width = 6)
grButton.place(x=int(176*g.xscale), y=int(410*g.yscale))

grButton.bind("<Button-3>", gr_right_click)

npButton = Button(canvas, text="Prev Page", font=('Arial',12), command = prevPage, bg = "bisque2", height = 1, width = 6)
npButton.place(x=int(254*g.xscale), y=int(410*g.yscale))

ppButton = Button(canvas, text="Next Page", font=('Arial',12), command = nextPage, bg = "bisque2", height = 1, width = 6)
ppButton.place(x=int(332*g.xscale), y=int(410*g.yscale))

modeButton = Button(canvas, text="AM", font=('Arial',12), command = Mode, bg = "bisque2", height = 1, width = 6)
modeButton.place(x=int(410*g.xscale), y=int(410*g.yscale))

keyerButton = Button(canvas, text="Iambic B", font=('Arial',12), command = Keyer, bg = "bisque2", height = 1, width = 6)
keyerButton.place(x=int(488*g.xscale), y=int(410*g.yscale))

saveButton = Button(canvas, text="Save Freq", font=('Arial',12), command = saveFreq, bg = "bisque2", height = 1, width = 6)
saveButton.place(x=int(566*g.xscale), y=int(410*g.yscale))

freqincButton = Button(canvas, text="Inc 10000", font=('Arial',12), command = freqInc, bg = "bisque2", height = 1, width = 6)
freqincButton.place(x=int(644*g.xscale), y=int(410*g.yscale))

freqincButton.bind("<Button-3>", fi_right_click)

shutdownButton = Button(canvas, text="Exit", font=('Arial',12), command = leave, bg = "red", fg = "bisque2", height = 1, width = 6)
shutdownButton.place(x=int(722*g.xscale), y=int(410*g.yscale))

#canvas.addtag_all("all")

# Set up mouse click event
win.bind('<Button-1>', canvas_click)

# Now set up an interrupt handler for LDAV going low
GPIO.add_event_detect(26,GPIO.BOTH,callback=LDAV)
	
LDAV_guts()	# complete HS if one has started

g.nrun = 1	# force read of pot data

# start the read_sdr thread
g.stopped = False
read_signals = threading.Thread(target=read_sdr_thread, args=(sdr,))
read_signals.start()
		
win.mainloop()

